<?php

(defined('BASEPATH')) OR exit('No direct script access allowed');

/**
 * Image CMS
 * Module Frame
 */
class Payment_method_payanyway extends MY_Controller
{

    public $paymentMethod;

    public $moduleName = 'payment_method_payanyway';

    public function __construct() {
        parent::__construct();
        $lang = new MY_Lang();
        $lang->load('payment_method_payanyway');
    }

    public function index() {
        lang('payanyway','payment_method_payanyway');
    }

    /**
     * Вытягивает данные способа оплаты
     * @param str $key
     * @return array
     */
    private function getPaymentSettings($key) {          
        $ci = &get_instance();
        $value = $ci->db->where('name', $key)
            ->get('shop_settings');
        if ($value) {
            $value = $value->row()->value;
        } else {
            show_error($ci->db->_error_message());
        }               
        return unserialize($value);
    }

    /**
     * @param $name
     * @return mixed
     */
    private function getPaymentMethodByName($name) {
        $ci = &get_instance();
        $value = $ci->db->where('payment_system_name', $name)
            ->get('shop_payment_methods');

        if ($value) {
            $value = $value->row()->id;
        } else {
            show_error($ci->db->_error_message());
        }
        return $value;
    }

    /**
     * Вызывается при редактировании способов оплатыв админке
     * @param int $id ид метода оплаты
     * @param string $payName название payment_method_payanyway
     * @return string
     */
    public function getAdminForm($id, $payName = null) {
        if (!$this->dx_auth->is_admin()){
            redirect('/');
            exit;
        }
        
        $nameMethod = $payName ? $payName : $this->paymentMethod->getPaymentSystemName();
        $key = $id . '_' . $nameMethod;
        $data = $this->getPaymentSettings($key);

        return '           
             <div class="control-group">
                <label class="control-label" for="inputRecCount">' . lang('Account', 'payment_method_payanyway') . ':</label>
                <div class="controls">
                  <input type="text" name="payment_method_payanyway[mnt_id]" value="' . $data['mnt_id'] . '"  />
                </div>
            </div>          
            <div class="control-group">
                <label class="control-label" for="inputRecCount">' . lang('Data integrity code', 'payment_method_payanyway') . ':</label>
                <div class="controls">
                 <input type="text" name="payment_method_payanyway[mnt_dataintegrity_code]" value="' . $data['mnt_dataintegrity_code'] . '"  />
                </div>
            </div>

            <div class="control-group">
                <label class="control-label" for="inputRecCount">' . lang('Merchant settings', 'payment_method_payanyway') . ':</label>
                <div class="controls" style="width:100%;">
                    <b>Pay URL:</b> ' . site_url('payment_method_payanyway/payment_method_payanyway/callback') . '<br/>
                    <span class="help-block">' . lang('The method of sending data for all requests: GET', 'main') . '</span>
                </div>
            </div>
   
        ';
    }

    /**
     * Формирование кнопки оплаты
     * @param obj $param Данные о заказе
     * @return str
     */
    public function getForm($param) {

        include_once(__DIR__ . "/moneta_sdk_lib/autoload.php");
        $monetaSDK = new \Moneta\MonetaSdk();

        $payment_method_id = $param->getPaymentMethod();
        $key = $payment_method_id . '_' . $this->moduleName;
        $paySettings = $this->getPaymentSettings($key);
	    // номер заказа
	    $order_id = $param->getId();
        // номер счёта получателя
	    $mnt_id = $paySettings['mnt_id'];
        $monetaSDK->setSettingValue('monetasdk_account_id', $mnt_id);
        // секретный код для формирования подписи
	    $mnt_dataintegrity_code = $paySettings['mnt_dataintegrity_code'];
        $monetaSDK->setSettingValue('monetasdk_account_code', $mnt_dataintegrity_code);
	    $mnt_description = "Оплата заказа номер " . $param->getId();
        $productsPrice = $param->getTotalPrice();
        $deliveryPrice = $param->SDeliveryMethods->getPrice();
	    $amount = ShopCore::app()->SCurrencyHelper->convert($deliveryPrice + $productsPrice);
	    $ISOCode = SCurrenciesQuery::create()->filterByIsDefault(true)->findOne()->getCode();
	    if ($ISOCode == 'RUR') {
            $ISOCode = 'RUB';
        }

        $result = $monetaSDK->showPaymentFrom($order_id, $amount, $ISOCode, $mnt_description);

        return $result->render;

    }

    const paw_kassa_VAT0     = '1104';  // НДС 0%
    const paw_kassa_VAT10    = '1103';  // НДС 10%
    const paw_kassa_VAT18    = '1102';  // НДС 18%
    const paw_kassa_VATNOVAT = '1105';  // НДС не облагается
    const paw_kassa_VATWR10  = '1107';  // НДС с рассч. ставкой 10%
    const paw_kassa_VATWR18  = '1106';  // НДС с рассч. ставкой 18%

    private static function monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, $success = false,
                                                 $repeatRequest = false, $echo = true, $die = true, $kassa_inventory = null,
                                                 $kassa_customer = null, $kassa_delivery = null)
    {
        if ($success === true)
            $resultCode = '200';
        elseif ($repeatRequest === true)
            $resultCode = '402';
        else
            $resultCode = '500';
        $mnt_signature = md5($resultCode.$mnt_id.$mnt_transaction_id.$mnt_data_integrity_code);
        $response = '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
        $response .= "<MNT_RESPONSE>\n";
        $response .= "<MNT_ID>{$mnt_id}</MNT_ID>\n";
        $response .= "<MNT_TRANSACTION_ID>{$mnt_transaction_id}</MNT_TRANSACTION_ID>\n";
        $response .= "<MNT_RESULT_CODE>{$resultCode}</MNT_RESULT_CODE>\n";
        $response .= "<MNT_SIGNATURE>{$mnt_signature}</MNT_SIGNATURE>\n";
        if (!empty($kassa_inventory) || !empty($kassa_customer) || !empty($kassa_delivery))
        {
            $response .= "<MNT_ATTRIBUTES>\n";
            foreach (array('INVENTORY' => $kassa_inventory, 'CUSTOMER' => $kassa_customer, 'DELIVERY' => $kassa_delivery) as $k => $v)
                if (!empty($v))
                    $response .= "<ATTRIBUTE><KEY>{$k}</KEY><VALUE>{$v}</VALUE></ATTRIBUTE>\n";
            $response .= "</MNT_ATTRIBUTES>\n";
        }
        $response .= "</MNT_RESPONSE>\n";
        if ($echo === true)
        {
            header("Content-type: application/xml");
            echo $response;
        }
        else
            return $response;
        if ($die === true)
            die;
        return '';
    }

    /**
     * Метод куда система шлет статус заказа
     */
    public function callback() {

        include_once(__DIR__ . "/moneta_sdk_lib/autoload.php");
        $monetaSDK = new \Moneta\MonetaSdk();

        $payment_method_id = $this->getPaymentMethodByName('payment_method_payanyway');
        if (!$payment_method_id) {
            die('FAIL');
        }

        $key = $payment_method_id . '_' . $this->moduleName;
        $paySettings = $this->getPaymentSettings($key);

        // номер счёта получателя
        $mnt_id = $paySettings['mnt_id'];
        $monetaSDK->setSettingValue('monetasdk_account_id', $mnt_id);
        // секретный код для формирования подписи
        $mnt_dataintegrity_code = $paySettings['mnt_dataintegrity_code'];
        $monetaSDK->setSettingValue('monetasdk_account_code', $mnt_dataintegrity_code);

        $result = $monetaSDK->processInputData();
        $data = $result->data;

        if (isset($data['processResultData']) && isset($data['processResultData']['orderId']))
        {
            $orderId = $data['processResultData']['orderId'];
            $this->checkPaid($orderId);

            $ci = &get_instance();
            $products = $ci->db->where('order_id', $orderId)->get('shop_orders_products');
            $order = $ci->db->where('id', $orderId)->get('shop_orders');

            if ($products && $order)
            {
                $items = $products->result_array();
                $user_email = $order->row()->user_email;
                $kassa_delivery = floatval($_REQUEST['MNT_AMOUNT']) - floatval($order->row()->total_price); //->delivery_price;
                $inventoryPositions = [];
                foreach ($items as $item)
                {
                    $inventoryPositions[] = [
                        'name' => $item['product_name'] . ' ' . $item['variant_name'],
                        'price' => $item['price'],
                        'quantity' => $item['quantity'],
                        'vatTag' => self::paw_kassa_VATNOVAT
                    ];
                }
                if ($kassa_delivery <= 0)
                    $kassa_delivery = null;
                $kassa_inventory = json_encode($inventoryPositions);
                self::monetaPayURLResponse($mnt_id, $_REQUEST['MNT_TRANSACTION_ID'], $mnt_dataintegrity_code, true,
                    false, true, true, $kassa_inventory, $user_email, $kassa_delivery);                         // 200
            }
        }

        die($result->render);

    }

    /**
     * @param $order_id
     * @return bool
     */
    private function checkPaid($order_id) {

        // TODO: переписать - НЕ РАБОТАЕТ!!! Код взят от робокассы

        $ci = &get_instance();

        $userOrder = $ci->db->where('id', $order_id)
                ->get('shop_orders');

        if ($userOrder) {
            $userOrder = $userOrder->row();
        } else {
            show_error($ci->db->_error_message());
        } 

        // Set order paid
        $this->successPaid($order_id, $userOrder);

        return true;
    }

	/**
	 * Save settings
	 *
	 * @param SPaymentMethods $paymentMethod
	 * @return bool|string
	 */
    public function saveSettings(SPaymentMethods $paymentMethod) {
        $saveKey = $paymentMethod->getId() . '_'.$this->moduleName;
        \ShopCore::app()->SSettings->set($saveKey, serialize($_POST['payment_method_payanyway']));

        return true;
    }

    /**
     * Переводит статус заказа в оплачено, и прибавляет пользователю
     * оплеченную сумму к акаунту
     * @param int $order_id ид заказа который обрабатывается
     * @param obj $userOrder данные заказа
     */
    private function successPaid($order_id, $userOrder) {  
        $ci = &get_instance();
        $amount = $ci->db->select('amout')
                        ->get_where('users', array('id' => $userOrder->user_id));
        
        if($amount){
            $amount = $amount->row()->amout;
        } else {
            show_error($ci->db->_error_message());
        }             
        $amount += $userOrder->total_price;      
        
        $result = $ci->db->where('id',$order_id)
                ->update('shop_orders', array('paid'=>'1'));
        if(!$result){
            show_error($ci->db->_error_message());
        }
        
        $result = $ci->db
                ->where('id', $userOrder->user_id)
                ->limit(1)
                ->update('users', array(
                    'amout' => str_replace(',', '.', $amount)
        ));
        if(!$result){
            show_error($ci->db->_error_message());
        }
    }

    public function autoload() {
        
    }

    public function _install() {  
        $ci = &get_instance();
        
        $result = $ci->db->where('name', $this->moduleName)
                ->update('components', array('enabled' => '1'));
        if(!$result){
            show_error($ci->db->_error_message());
        }
    }

    public function _deinstall() {  
        $ci = &get_instance();
        
        $result = $ci->db->where('payment_system_name', $this->moduleName)
                        ->update('shop_payment_methods', array(
                                'active'=>'0',
                                'payment_system_name'=>'0',
                                ));
        if(!$result){
            show_error($ci->db->_error_message());
        }
        
        $result = $ci->db->like('name', $this->moduleName)
                        ->delete('shop_settings');
        if(!$result){
            show_error($ci->db->_error_message());
        }
        
    }

}

/* End of file sample_module.php */
